from fastapi import (
    APIRouter,
    Depends,
    HTTPException,
    status,
)


from sqlalchemy.ext.asyncio import AsyncConnection

from api.db.connection.session import get_connection_dep

from api.db.logic.account import (
    get_user_by_id,
    update_user_by_id,
    create_user,
    get_user_by_login,
    get_user_accaunt_page,
)

from api.schemas.account.account import User
from api.db.tables.account import AccountStatus
from api.schemas.base import bearer_schema
from api.schemas.endpoints.account import UserUpdate, AllUserResponse
from api.services.auth import get_current_user

from api.services.user_role_validation import db_user_role_validation
from api.services.update_data_validation import update_user_data_changes


api_router = APIRouter(
    prefix="/account",
    tags=["User accountModel"],
)


@api_router.get("", dependencies=[Depends(bearer_schema)], response_model=AllUserResponse)
async def get_all_account(
    page: int = 1,
    limit: int = 10,
    connection: AsyncConnection = Depends(get_connection_dep),
    current_user=Depends(get_current_user),
):
    authorize_user = await db_user_role_validation(connection, current_user)

    user_list = await get_user_accaunt_page(connection, page, limit)

    if user_list is None:
        raise HTTPException(status_code=status.HTTP_404_NOT_FOUND, detail="Accounts not found")

    return user_list


@api_router.get("/{user_id}", dependencies=[Depends(bearer_schema)], response_model=User)
async def get_account(
    user_id: int, connection: AsyncConnection = Depends(get_connection_dep), current_user=Depends(get_current_user)
):
    authorize_user = await db_user_role_validation(connection, current_user)

    user = await get_user_by_id(connection, user_id)

    if user is None:
        raise HTTPException(status_code=status.HTTP_404_NOT_FOUND, detail="Account not found")

    return user


@api_router.post("", dependencies=[Depends(bearer_schema)], response_model=User)
async def create_account(
    user: UserUpdate, connection: AsyncConnection = Depends(get_connection_dep), current_user=Depends(get_current_user)
):
    authorize_user = await db_user_role_validation(connection, current_user)

    user_validation = await get_user_by_login(connection, user.login)

    if user_validation is None:
        await create_user(connection, user, authorize_user.id)
        user_new = await get_user_by_login(connection, user.login)
        return user_new

    else:
        raise HTTPException(
            status_code=status.HTTP_400_BAD_REQUEST, detail="An account with this information already exists."
        )


@api_router.put("/{user_id}", dependencies=[Depends(bearer_schema)], response_model=User)
async def update_account(
    user_id: int,
    user_update: UserUpdate,
    connection: AsyncConnection = Depends(get_connection_dep),
    current_user=Depends(get_current_user),
):
    authorize_user = await db_user_role_validation(connection, current_user)

    user = await get_user_by_id(connection, user_id)
    if user is None:
        raise HTTPException(status_code=status.HTTP_404_NOT_FOUND, detail="Account not found")

    update_values = update_user_data_changes(user_update, user)

    if update_values is None:
        return user

    user_update_data = User.model_validate({**user.model_dump(), **update_values})

    await update_user_by_id(connection, update_values, user)

    user = await get_user_by_id(connection, user_id)

    return user


@api_router.delete("/{user_id}", dependencies=[Depends(bearer_schema)], response_model=User)
async def delete_account(
    user_id: int, connection: AsyncConnection = Depends(get_connection_dep), current_user=Depends(get_current_user)
):
    authorize_user = await db_user_role_validation(connection, current_user)

    user = await get_user_by_id(connection, user_id)
    if user is None:
        raise HTTPException(status_code=status.HTTP_404_NOT_FOUND, detail="Account not found")

    user_update = UserUpdate(status=AccountStatus.DELETED.value)

    update_values = update_user_data_changes(user_update, user)

    if update_values is None:
        return user

    await update_user_by_id(connection, update_values, user)

    user = await get_user_by_id(connection, user_id)

    return user
